/*  Copyright 2013, Unpublished Work of Technologic Systems
*  All Rights Reserved.
*
*  THIS WORK IS AN UNPUBLISHED WORK AND CONTAINS CONFIDENTIAL,
*  PROPRIETARY AND TRADE SECRET INFORMATION OF TECHNOLOGIC SYSTEMS.
*  ACCESS TO THIS WORK IS RESTRICTED TO (I) TECHNOLOGIC SYSTEMS EMPLOYEES
*  WHO HAVE A NEED TO KNOW TO PERFORM TASKS WITHIN THE SCOPE OF THEIR
*  ASSIGNMENTS  AND (II) ENTITIES OTHER THAN TECHNOLOGIC SYSTEMS WHO
*  HAVE ENTERED INTO  APPROPRIATE LICENSE AGREEMENTS.  NO PART OF THIS
*  WORK MAY BE USED, PRACTICED, PERFORMED, COPIED, DISTRIBUTED, REVISED,
*  MODIFIED, TRANSLATED, ABRIDGED, CONDENSED, EXPANDED, COLLECTED,
*  COMPILED,LINKED,RECAST, TRANSFORMED, ADAPTED IN ANY FORM OR BY ANY
*  MEANS,MANUAL, MECHANICAL, CHEMICAL, ELECTRICAL, ELECTRONIC, OPTICAL,
*  BIOLOGICAL, OR OTHERWISE WITHOUT THE PRIOR WRITTEN PERMISSION AND
*  CONSENT OF TECHNOLOGIC SYSTEMS . ANY USE OR EXPLOITATION OF THIS WORK
*  WITHOUT THE PRIOR WRITTEN CONSENT OF TECHNOLOGIC SYSTEMS  COULD
*  SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.
*/
/* To compile imx28_adc, use the appropriate cross compiler and run the
* command:
*
*  gcc -fno-tree-cselim -Wall -O -mcpu=arm9 -o imx28_adc imx28_adc.c
*
* On uclibc based initrd's, the following additional gcc options are
* necessary: -Wl,--rpath,/slib -Wl,-dynamic-linker,/slib/ld-uClibc.so.0
*
* This program is used to read and average the i.MX28 CPU LRADC channels
* 1, 2, 3, 4, and 6 and report their reading in milivolts.  
* This program allows all of the ADCs to be calibrated with one value.
* To use, simply apply a known voltage to the ADC pins, take the difference
* of the known voltage and the read voltage in mV, and set the CAL #define
* below to the calculated difference.
*
* Not all i.MX28 based SBCs use all channels, please see the documentation
* of the respective product for information on what channels are in use and
* what they are connected to.
*/
#include <assert.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/mman.h>
#include <unistd.h>

#define CAL 0  //Calibration offset in mv

int main(void) {
	volatile unsigned int *mxlradcregs;
	int devmem = 0, x, chan[5] = {0,0,0,0,0};
	int i;

	devmem = open("/dev/mem", O_RDWR|O_SYNC);
	assert(devmem);

	mxlradcregs = (unsigned int *) mmap(0, getpagesize(),
	  PROT_READ | PROT_WRITE, MAP_SHARED, devmem, 0x80050000);

	mxlradcregs[0x148/4] = 0xfffff; //Clear LRADC4:0 assignments
	mxlradcregs[0x144/4] = 0x64321; //Set LRDAC4:0 to channel 6,4:1
	for(x = 0; x < 5; x++)
	  mxlradcregs[(0x50+(x * 0x10))/4] = 0x0; //Clear LRADCx reg
	
	for(x = 0; x < 10; x++) {
		mxlradcregs[0x18/4] = 0x1f; //Clear interrupt ready
		mxlradcregs[0x4/4] = 0x1f; //Schedule conversaion of chan 4:0
		while(!((mxlradcregs[0x10/4] & 0x1f) == 0x1f)); //Wait 
		for(i = 0; i < 5; i++) 
		  chan[i] += (mxlradcregs[(0x50+(i * 0x10))/4] & 0xffff);
	}


	for(x = 0; x < 5; x++) {
		/* Multiply by 1000 to get mV, divide by 10 samples
		 * Analog dividers are divide by two on ADC 1-4, 
		 * 6 is divide by 3
		 *
		 * Calibration offsets should be added here */
		if(x == 4)
		  chan[x] = ((((chan[x] / 10) * 45177) / 1000000) * 33) + CAL;
		else
		  chan[x] = ((((chan[x] / 10) * 45177) / 100000) * 2) + CAL;
		printf("LRADC_ADC%d_millivolts=%d\n", x, chan[x]);
	}

	return 0;
}
